from django.contrib.contenttypes.models import ContentType

from popularity import settings, models, utils

class Base(object):
    """
    Base backend for managing objects scores.
    The management commands will use the following methods:
        - hit(obj, points=1)
        - update(period)
    """
    def __init__(self):
        self._rank_cache = {}
        self._content_type_cache = {}
        
    def _save_all_ranks(self):
        for rank in self._rank_cache.values():
            rank.save()
            
    def pre_weekly_update(self):
        models.Rank.objects.all().update(weekly=settings.CAP, absolute=settings.CAP)
        
    def post_weekly_update(self):
        pass
        
    def pre_monthly_update(self):
        models.Rank.objects.all().update(monthly=settings.CAP)
        
    def post_monthly_update(self):
        pass
    
    def hit(self, obj, points=1):
        model_name = str(obj._meta)
        if model_name in settings.RANKABLE_MODELS:
            content_type = self._content_type_cache.get(model_name)
            if content_type is None:
                app_label, model = model_name.lower().split(".")
                content_type = ContentType.objects.get(
                    app_label=app_label, model=model)
                self._content_type_cache[model_name] = content_type
            self.hit_content(content_type, obj, points)
      
    def update(self, period):
        """
        Period must be *weekly* or "monthly".
        """
        getattr(self, "pre_%s_update" % period)()
        lookup = utils.get_content_type_lookup()
        for content_type in ContentType.objects.filter(lookup):
            getattr(self, "update_%s" % period)(content_type)
        self._save_all_ranks()
        getattr(self, "post_%s_update" % period)()
    
    def update_rank(self, content_type, object_id, **kwargs):
        rank = self._rank_cache.get((content_type, object_id))
        if rank is None:
            rank, created = models.Rank.objects.get_or_create(
                content_type=content_type, object_id=object_id)
            self._rank_cache[(content_type, object_id)] = rank
        for k, v in kwargs.items():
            setattr(rank, k, v)
        if "weekly" in kwargs:
            rank.update_history(kwargs["weekly"])
        return rank
   

class Django(Base):
    """
    Database backend using a django model to store scores.
    """
    def hit_content(self, content_type, obj, points=1):
        score, created = models.Score.objects.get_or_create(
            content_type=content_type, object_id=obj.pk)
        score.hit(points)
        
    def _update_absolute(self, content_type):
        scores = models.Score.objects.filter(content_type=content_type)
        for num, score in enumerate(scores.order_by("-absolute")[:settings.CAP]):
            self.update_rank(content_type, score.object_id, absolute=num+1)
    
    def update_weekly(self, content_type):
        self._update_absolute(content_type)
        scores = models.Score.objects.filter(content_type=content_type)
        for num, score in enumerate(scores.order_by("-weekly")[:settings.CAP]):
            self.update_rank(content_type, score.object_id, weekly=num+1)

    def post_weekly_update(self):
        models.Score.objects.all().update(weekly=0)
        
    def update_monthly(self, content_type):
        scores = models.Score.objects.filter(content_type=content_type)
        for num, score in enumerate(scores.order_by("-monthly")[:settings.CAP]):
            self.update_rank(content_type, score.object_id, monthly=num+1)

    def post_monthly_update(self):
        models.Score.objects.all().update(monthly=0)
        
        
class Mongodb(Base):
    """
    Fast backend using mongodb collections to store scores.
    """    
    def _get_document(self, content_type):
        from mongoengine import Document, IntField
        collection = "score_%s_%s" % (content_type.app_label, content_type.model)
        
        class Score(Document):
            object_id = IntField(required=True, min_value=1, unique=True)
            weekly = IntField(min_value=0)
            monthly = IntField(min_value=0)
            absolute = IntField(min_value=0)

            meta = {
                'indexes': ['object_id'],
                'collection': collection,
            }
            
        return Score
    
    @utils.mongo_connect
    def hit_content(self, content_type, obj, points=1):
        document = self._get_document(content_type)
        document.objects(object_id=obj.pk).update_one(upsert=True, 
            inc__weekly=points, inc__monthly=points, inc__absolute=points)
        
    def _update_absolute(self, content_type, document):
        for num, score in enumerate(document.objects.order_by("-absolute")[:settings.CAP]):
            self.update_rank(content_type, score.object_id, absolute=num+1)
    
    @utils.mongo_connect
    def update_weekly(self, content_type):
        document = self._get_document(content_type)
        self._update_absolute(content_type, document)
        for num, score in enumerate(document.objects.order_by("-weekly")[:settings.CAP]):
            self.update_rank(content_type, score.object_id, weekly=num+1)
        document.objects.update(set__weekly=0)
    
    @utils.mongo_connect
    def update_monthly(self, content_type):
        document = self._get_document(content_type)
        for num, score in enumerate(document.objects.order_by("-monthly")[:settings.CAP]):
            self.update_rank(content_type, score.object_id, monthly=num+1)
        document.objects.update(set__monthly=0)
