import datetime

from django.db import models
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.utils.datastructures import SortedDict

from popularity import settings, utils, json

class History(SortedDict):
    """
    A rank history.
    """
    def __init__(self, *args, **kwargs):
        super(History, self).__init__(*args, **kwargs)
        self.current_sunday = self._get_sunday_for(datetime.date.today())
    
    def _get_sunday_for(self, date):
        return date - datetime.timedelta(days=date.weekday()+1)
                
    def items_from_date(self, date):
        date = self._get_sunday_for(date)
        while date <= self.current_sunday:
            value = self.get(date, settings.CAP)
            yield date, value
            date += datetime.timedelta(days=7)
            
    def is_updated(self):
        return self.keys()[-1] == self.current_sunday
        
    def current_rank(self):
        return self.get(self.current_sunday, settings.CAP)
    
    def previous_rank(self):
        date = self.current_sunday - datetime.timedelta(days=7)
        return self.get(date, settings.CAP)
        
    def trend(self):
        return self.previous_rank() - self.current_rank()
        
    def set_current(self, value):
        self[self.current_sunday] = value
        
                
class Rank(models.Model):
    """
    Rank for a generic object.
    """
    content_type = models.ForeignKey(ContentType,
        limit_choices_to=utils.get_content_type_lookup())
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    
    weekly = models.PositiveIntegerField(default=settings.CAP)
    monthly = models.PositiveIntegerField(default=settings.CAP)
    absolute = models.PositiveIntegerField(default=settings.CAP)
    
    history = json.JsonField(blank=True)
    
    class Meta:
        unique_together = ("content_type", "object_id")

    def __unicode__(self):
        return u"Rank for %s" % self.content_object
    
    def get_history(self):
        """
        Return a version of the history containing all the weeks
        till today.
        """
        history = self.history or []
        unserialize = lambda x: datetime.date(*[int(i) for i in x.split("-")])
        items = [(unserialize(k), v) for k, v in history]
        return History(items)
        
    def set_history(self, history):
        self.history = [(str(k), v) for k, v in history.items()]
        
    def update_history(self, position):
        history = self.get_history()
        history.set_current(position)
        self.set_history(history)
        
    def trend(self):
        """
        Return the object rank trend between current and passed week.
        """
        return self.get_history().trend()
        
    def get_trend_display(self):
        """
        Return the trend as string: "positive", "negative" or "same".
        """
        trend = self.trend()
        if trend > 0:
            return u"positive"
        if trend < 0:
            return u"negative"
        return u"same"
        

class RankableModel(models.Model):
    """
    Mixin for models with a rank.
    """
    ranks = generic.GenericRelation(Rank)
    
    class Meta:
        abstract = True 
    
    @property
    def rank(self):
        return self.ranks.get()


# if provided django backend is used than necessary model must be defined
if settings.BACKEND == 'popularity.backends.Django':
     
    class Score(models.Model):
        """
        Score for a generic object.
        """
        content_type = models.ForeignKey(ContentType)
        object_id = models.PositiveIntegerField()
        content_object = generic.GenericForeignKey('content_type', 'object_id')

        weekly = models.PositiveIntegerField(default=0)
        monthly = models.PositiveIntegerField(default=0)
        absolute = models.PositiveIntegerField(default=0)

        class Meta:
            unique_together = ("content_type", "object_id")

        def __unicode__(self):
            return u"Score for %s" % self.content_object

        def hit(self, points=1):
            queryset = self.__class__.objects.filter(pk=self.pk)
            queryset.update(
                weekly=models.F("weekly") + points,
                monthly=models.F("monthly") + points,
                absolute=models.F("absolute") + points,
            )
