from django import template

from popularity import settings, shortcuts, utils

register = template.Library()

@register.tag
def popularity_hit(parser, token, use_ajax=False):
    """
    Update popularity score for passed object(s)::
        
        {% popularity_hit object %}
        
    You can set score points (default is 1)::
      
        {% popularity_hit object=3 %}
        
    And you can hit several objects::
      
        {% popularity_hit object=2 object.user another_object=5 %}
    """
    try:
        tag_name, arg = token.contents.split(None, 1)
    except ValueError:
        error = u"%r tag requires arguments" % token.contents.split()[0]
    str_points_map = {}
    for i in arg.split():
        obj, points = i.split("=") if ("=" in i) else (i, 1)
        str_points_map[obj] = int(points)
    return PopularityNode(use_ajax, str_points_map)
    
@register.tag
def popularity_request(parser, token):
    """
    Same as *popularity_hit* but updates the score using an ajax request.
    """
    return popularity_hit(parser, token, use_ajax=True)
    
class PopularityNode(template.Node):
    def __init__(self, use_ajax, str_points_map, 
        template_name="popularity/popularity_request.html"):
        self.use_ajax = use_ajax
        self.var_points = ((template.Variable(k), v) for k, v in str_points_map.items())
        self.template_name = template_name

    def render(self, context):
        if settings.BACKEND is None:
            return u''
        obj_points = [(k.resolve(context), v) for k, v in self.var_points]
        if self.use_ajax:
            context = {"objects": utils.serialize(obj_points)}
            return template.loader.render_to_string(self.template_name, context)
        else:
            request = context.get("request")
            for obj, points in obj_points:
                shortcuts.hit(obj, points, request=request)
            return u''
            
            
@register.tag
def get_most_popular(parser, token):
    """
    Return a queryset ordered by popularity rank.

    Usage::
    
        {% get_most_popular [model] [period] as [varname] %}
        
    The argument *period* must be *weekly*, *monthly* or *absolute*.
    The argument *model* must be a dotted model name, a model class or a queryset.
    
    Examples::
    
        {% get_most_popular 'auth.user' weekly as weekly_popular_users %}
        {% get_most_popular User monthly as monthly_popular_users %}
    """
    # args validation
    try:
        tag_name, model, period, as_, varname = token.contents.split()
    except ValueError:
        message = "Invalid arguments for %r tag" % token.contents.split()[0]
        raise template.TemplateSyntaxError, message
    if period not in ("weekly", "monthly", "absolute"):
        message = "Invalid period %s for %r tag" % (tag_name, period)
        raise template.TemplateSyntaxError, message
    return GetMostPopularNode(model, period, varname)
    
class GetMostPopularNode(template.Node):
    def __init__(self, model, period, varname):
        # model
        self.model_variable = None
        if model[0] in ('"', "'") and model[-1] == model[0]:
            self.model_name = model[1:-1]
        else:
            self.model_variable = template.Variable(model)
        # period
        self.period = period
        # varname
        self.varname = varname
    
    def render(self, context):
        # model and queryset
        if self.model_variable is None:
            queryset = shortcuts.get_queryset(self.model_name)
        else:
            model = self.model_variable.resolve(context)
            try:
                model_name = str(model._meta)
            except AttributeError:
                queryset = shortcuts.patch(model)
            else:
                queryset = shortcuts.get_queryset(model_name)            
        # context
        context[self.varname] = queryset.order_by("ranks__%s" % self.period)
        return u""
        
        
@register.inclusion_tag("popularity/show_most_popular.html", takes_context=True)
def show_most_popular(context, limit, model, period=None):
    """
    Show *limit* number of the most popular instances of *model*.
    The argument *model* can be a dotted model name, a model class or a queryset.
    The argument *period* must be *weekly*, *monthly* or *absolute* and two
    periods can be specified using comma (e.g.:: *weekly,absolute*).
    If *period* is None than popular instances of all periods are displayed.
    """
    if isinstance(model, basestring):
        queryset = shortcuts.get_queryset(model)
    else:
        try:
            model_name = str(model._meta)
        except AttributeError:
            queryset = shortcuts.patch(model)
        else:
            queryset = shortcuts.get_queryset(model_name)
    periods = ("weekly", "monthly", "absolute") if period is None else period.split(",")
    context['periods'] = periods
    for i in periods:
        context["%s_popular" % i] = queryset.order_by("ranks__%s" % i)[:limit]
    return context
