try:
    from importlib import import_module
except ImportError:
    from django.utils.importlib import import_module

from django.db.models import get_model, Q
from django.core.exceptions import ImproperlyConfigured
from django.utils import simplejson as json
    
from popularity import settings

def load_object(path):
    """
    Return the python object represented by dotted *path*.
    """
    i = path.rfind('.')
    module, attr = path[:i], path[i+1:]
    try:
        mod = import_module(module)
    except ImportError, err:
        message = 'Error loading object %s: "%s"'
        raise ImproperlyConfigured(message % (module, err))
    try:
        obj = getattr(mod, attr)
    except AttributeError:
        message = 'Module "%s" does not define an object named "%s"'
        raise ImproperlyConfigured(message % (module, attr))
    return obj
    
def get_content_type_lookup():
    q = Q()
    for model_name in settings.RANKABLE_MODELS:
        app_label, model = model_name.lower().split(".")
        q = q | Q(**{'app_label': app_label, 'model': model})
    return q

def get_backend():
    """
    Return the backend used to store scores.
    """
    if settings.BACKEND is None:
        
        class Dummy(object):
            """
            Dummy backend that do nothing.
            """
            def hit(self, obj, points=1):
                pass

            def update(self, period):
                pass
        
        return Dummy()

    return load_object(settings.BACKEND)()
    
def get_handler():
    """
    Return the hit handler that will manage score assignment.
    Th handler is a callable that will be called passing:
        - the popularity backend instance
        - the request
        - the object to hit
        - the score points
    """
    return load_object(settings.HANDLER)
    
def mongo_connect(function):
    """
    Decorator to extablish a connection with mongodb passed *dbname*.
    If there is a problem during connection the decorated view will do nothing.
    """
    name = settings.MONGODB["NAME"]
    username = settings.MONGODB.get("USERNAME")
    password = settings.MONGODB.get("PASSWORD")
    parameters = settings.MONGODB.get("PARAMETERS", {})
    def decorated(*args, **kwargs):
        import mongoengine
        try:
            mongoengine.connect(name, username, password, **parameters)
        except mongoengine.ConnectionError, err:
            print "MongoDb connection error:", err
            return
        return function(*args, **kwargs)
    return decorated
    
def serialize(obj_points):
    """
    Serialize a sequence of tuples mapping model instances to points, e.g.::
        
        [(article, 2), (article, 1)] --> [["article.article", 1, 2], ["article.article", 12, 1]]
    """
    return json.dumps([(str(obj._meta), obj.pk, points) for obj, points in obj_points])

def deserialize(json_data):
    """
    Deserialize the data serialized with the above function.
    Return None if the *json_data* is invald.
    """
    try:
        model_pk_points = [
            (get_model(*model_name.split(".")), int(pk), int(points))
            for model_name, pk, points in json.loads(json_data)
        ]
    except (TypeError, ValueError):
        return None
    obj_points = []
    for model, pk, points in model_pk_points:
        if model is None:
            return None
        try:
            obj = model.objects.get(pk=pk)
        except model.DoesNotExist:
            return None
        obj_points.append((obj, points))
    return obj_points
    
def queryset_callback(model, patch):
    """
    Call the queryset callback used by *popularity.shortcuts.get_queryset*.
    """
    if settings.QUERYSET_CALLBACK is None:
        return patch(model.objects.all())
    return load_object(settings.QUERYSET_CALLBACK)(model, patch)
